<?php
declare(strict_types=1);

namespace CakeProducts\Controller;

use App\Controller\AppController;
use Cake\Core\Configure;
use Cake\ORM\Table;
use Cake\ORM\TableRegistry;
use CheeseCake\Controller\Traits\OverrideTableTrait;

/**
 * ProductCategoryVariants Controller
 *
 * @property \App\Model\Table\ProductCategoryVariantsTable $ProductCategoryVariants
 */
class ProductCategoryVariantsController extends AppController
{
    use OverrideTableTrait;

    /**
     * @return void
     */
    public function initialize(): void
    {
        parent::initialize(); // TODO: Change the autogenerated stub
//        $this->_defaultTable = 'CakeProducts.ProductCategoryVariants';
//        $this->_tableConfigKey = 'CakeProducts.ProductCategoryVariants.table';
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|null|void Renders view
     */
    public function index()
    {
        $query = $this->getTable()->find()
            ->contain(['ProductCategories', 'Products']);
        $productCategoryVariants = $this->paginate($query);

        $this->set(compact('productCategoryVariants'));
    }

    /**
     * View method
     *
     * @param string|null $id Product Category Variant id.
     * @return \Cake\Http\Response|null|void Renders view
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
        $productCategoryVariant = $this->getTable()->get($id, contain: ['ProductCategories', 'Products']);
        $this->set(compact('productCategoryVariant'));
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null|void Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $productCategoryVariantsTable = $this->getTable();

        $productCategoryVariant = $productCategoryVariantsTable->newEmptyEntity();
        if ($this->request->is('post')) {
            $productCategoryVariant = $productCategoryVariantsTable->patchEntity($productCategoryVariant, $this->request->getData());
            if ($productCategoryVariantsTable->save($productCategoryVariant)) {
                $this->Flash->success(__('The product category variant has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            $this->Flash->error(__('The product category variant could not be saved. Please, try again.'));
        }
        $productCategories = $productCategoryVariantsTable->ProductCategories->find('list', keyField: 'internal_id', valueField: 'name')->all();
        $products = $productCategoryVariantsTable->Products->find('list')->all();
        $this->set(compact('productCategoryVariant', 'productCategories', 'products'));
    }

    /**
     * Edit method
     *
     * @param string|null $id Product Category Variant id.
     * @return \Cake\Http\Response|null|void Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $productCategoryVariantsTable = $this->getTable();
        $productCategoryVariant = $productCategoryVariantsTable->get($id, contain: []);
        if ($this->request->is(['patch', 'post', 'put'])) {
            $productCategoryVariant = $productCategoryVariantsTable->patchEntity($productCategoryVariant, $this->request->getData());
            if ($productCategoryVariantsTable->save($productCategoryVariant)) {
                $this->Flash->success(__('The product category variant has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            $this->Flash->error(__('The product category variant could not be saved. Please, try again.'));
        }
        $productCategories = $productCategoryVariantsTable->ProductCategories->find('list', keyField: 'internal_id', valueField: 'name')->all();
        $products = $productCategoryVariantsTable->Products->find('list', limit: 200)->where(['product_category_id' => $productCategoryVariant->product_category_id])->all();
        $this->set(compact('productCategoryVariant', 'productCategories', 'products'));
    }

    /**
     * Delete method
     *
     * @param string|null $id Product Category Variant id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);
        $productCategoryVariantsTable = $this->getTable();

        $productCategoryVariant = $productCategoryVariantsTable->get($id);
        if ($productCategoryVariantsTable->delete($productCategoryVariant)) {
            $this->Flash->success(__('The product category variant has been deleted.'));
        } else {
            $this->Flash->error(__('The product category variant could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }

    /**
     * Gets the users table instance
     *
     * @return Table
     */
    public function getTable()
    {
        if ($this->_table instanceof Table) {
            return $this->_table;
        }
        $this->_table = TableRegistry::getTableLocator()->get(
            Configure::read('CakeProducts.ProductCategoryVariants.table', 'CakeProducts.ProductCategoryVariants')
        );

        return $this->_table;
    }
}
