<?php
declare(strict_types=1);

namespace CakeProducts\Controller;

use Cake\Log\Log;
use CakeProducts\Controller\AppController;
use CheeseCake\Controller\Traits\OverrideTableTrait;

/**
 * ProductSkus Controller
 *
 * @property \CakeProducts\Model\Table\ProductSkusTable $ProductSkus
 */
class ProductSkusController extends AppController
{
    use OverrideTableTrait;

    /**
     * @return void
     */
    public function initialize(): void
    {
        parent::initialize(); // TODO: Change the autogenerated stub
//        $this->_defaultTable = 'CakeProducts.ProductSkus';
//        $this->_tableConfigKey = 'CakeProducts.ProductSkus.table';
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|null|void Renders view
     */
    public function index()
    {
        $query = $this->ProductSkus->find()
            ->contain(['Products']);
        $productSkus = $this->paginate($query);

        $this->set(compact('productSkus'));
    }

    /**
     * View method
     *
     * @param string|null $id Product Skus id.
     * @return \Cake\Http\Response|null|void Renders view
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
        $productSku = $this->ProductSkus->get($id, contain: ['Products']);
        $this->set(compact('productSku'));
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null|void Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $productSku = $this->ProductSkus->newEmptyEntity();
        if ($this->request->is('post')) {
            $postData = $this->request->getData();
            $saveOptions = [
                'associated' => [],
            ];
            //            Log::debug(print_r('$postData', true));
//            Log::debug(print_r($postData, true));
//            Log::debug(print_r('$saveOptions', true));
//            Log::debug(print_r($saveOptions, true));
            $productSku = $this->ProductSkus->patchEntity($productSku, $postData, $saveOptions);
            if ($this->ProductSkus->save($productSku)) {
                $this->Flash->success(__('The product sku has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            Log::debug(print_r('$productSku->getErrors() next - failed in productSkus/add', true));
            Log::debug(print_r($productSku->getErrors(), true));
            $this->Flash->error(__('The product skus could not be saved. Please, try again.'));
        }
        $products = $this->ProductSkus->Products->find('list', limit: 200)->all();
        $this->set(compact('productSku', 'products'));
    }

    /**
     * Edit method
     *
     * @param string|null $id Product Skus id.
     * @return \Cake\Http\Response|null|void Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $productSku = $this->ProductSkus->get($id, contain: []);
        if ($this->request->is(['patch', 'post', 'put'])) {
            $postData = $this->request->getData();
            $saveOptions = [
                'associated' => [],
            ];
            //            Log::debug(print_r('$postData', true));
//            Log::debug(print_r($postData, true));
//            Log::debug(print_r('$saveOptions', true));
//            Log::debug(print_r($saveOptions, true));
            $productSku = $this->ProductSkus->patchEntity($productSku, $postData, $saveOptions);
            if ($this->ProductSkus->save($productSku)) {
                $this->Flash->success(__('The product skus has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            Log::debug(print_r('$productSku->getErrors() next - failed in productSkus/edit', true));
            Log::debug(print_r($productSku->getErrors(), true));
            $this->Flash->error(__('The product skus could not be saved. Please, try again.'));
        }
        $products = $this->ProductSkus->Products->find('list', limit: 200)->all();
        $this->set(compact('productSku', 'products'));
    }

    /**
     * Delete method
     *
     * @param string|null $id Product Skus id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);
        $productSku = $this->ProductSkus->get($id);
        if ($this->ProductSkus->delete($productSku)) {
            $this->Flash->success(__('The product skus has been deleted.'));
        } else {
            $this->Flash->error(__('The product skus could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }
}
