<?php
declare(strict_types=1);

namespace CakeProducts\Test\TestCase\Controller;

use CakeProducts\Controller\ProductCategoryVariantsController;
use Cake\TestSuite\IntegrationTestTrait;
use Cake\TestSuite\TestCase;
use CakeProducts\Model\Table\ProductCategoryVariantsTable;
use PHPUnit\Exception;

/**
 * CakeProducts\Controller\ProductCategoryVariantsController Test Case
 *
 * @uses \CakeProducts\Controller\ProductCategoryVariantsController
 */
class ProductCategoryVariantsControllerTest extends BaseControllerTest
{
    /**
     * Test subject
     *
     * @var ProductCategoryVariantsTable|Table
     */
    protected $ProductCategoryVariants;
    /**
     * Fixtures
     *
     * @var array<string>
     */
    protected array $fixtures = [
        'plugin.CakeProducts.ProductCategoryVariants',
        'plugin.CakeProducts.ProductCategories',
        'plugin.CakeProducts.Products',
    ];

    /**
     * setUp method
     *
     * @return void
     */
    protected function setUp(): void
    {
        parent::setUp();
        $this->ProductCategoryVariants = $this->getTableLocator()->get('ProductCategoryVariants');
    }

    /**
     * tearDown method
     *
     * @return void
     */
    protected function tearDown(): void
    {
        unset($this->ProductCategoryVariants);

        parent::tearDown();
    }

    /**
    * Test index method
    *
    * Tests the index action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCategoryVariantsController::index()
    * @throws Exception
    *
    * @return void
    */
    public function testIndexGet(): void
    {
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryVariants',
            'action' => 'index',
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test view method
    *
    * Tests the view action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCategoryVariantsController::view()
    * @throws Exception
    *
    * @return void
    */
    public function testViewGet(): void
    {
        $id = '5a386e9f-6e7a-4ae7-9360-c8e529f78d93';
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryVariants',
            'action' => 'view',
            $id,
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test add method
    *
    * Tests the add action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCategoryVariantsController::add()
    * @throws Exception
    *
    * @return void
    */
    public function testAddGet(): void
    {
        $cntBefore = $this->ProductCategoryVariants->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryVariants',
            'action' => 'add',
        ];
        $this->get($url);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCategoryVariants->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCategoryVariantsController::add()
    * @throws Exception
    *
    * @return void
    */
    public function testAddPostLoggedInSuccess(): void
    {
        $cntBefore = $this->ProductCategoryVariants->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryVariants',
            'action' => 'add',
        ];
        $data = [
            'name' => 'Size',
            'product_category_id' => 'db4b4273-eddc-46d4-93c8-45cf7c6e058e',
            'enabled' => true,
        ];
        $this->post($url, $data);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-category-variants');

        $cntAfter = $this->ProductCategoryVariants->find()->count();
        $this->assertEquals($cntBefore + 1, $cntAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCategoryVariantsController::add()
    * @throws Exception
    *
    * @return void
    */
    public function testAddPostLoggedInFailure(): void
    {
        $cntBefore = $this->ProductCategoryVariants->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryVariants',
            'action' => 'add',
        ];
        $data = [
            'name' => '',
            'product_category_id' => 'db4b4273-eddc-46d4-93c8-45cf7c6e058e',
            'enabled' => true,
        ];
        $this->post($url, $data);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCategoryVariants->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test edit method
    *
    * Tests the edit action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCategoryVariantsController::edit()
    * @throws Exception
    *
    * @return void
    */
    public function testEditGet(): void
    {
        $this->loginUserByRole('admin');
        $id = '5a386e9f-6e7a-4ae7-9360-c8e529f78d93';

        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryVariants',
            'action' => 'edit',
            $id,
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCategoryVariantsController::edit()
    * @throws Exception
    *
    * @return void
    */
    public function testEditPutLoggedInSuccess(): void
    {
        $this->loginUserByRole('admin');
        $id = '5a386e9f-6e7a-4ae7-9360-c8e529f78d93';
        $before = $this->ProductCategoryVariants->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryVariants',
            'action' => 'edit',
            $id,
        ];
        $data = [
            // test new data here
            'name' => 'updated name',
            'product_category_id' => 'db4b4273-eddc-46d4-93c8-45cf7c6e058e',
            'enabled' => true,
        ];
        $this->put($url, $data);

        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-category-variants');

        $after = $this->ProductCategoryVariants->get($id);
        // assert saved properly below
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCategoryVariantsController::edit()
    * @throws Exception
    *
    * @return void
    */
    public function testEditPutLoggedInFailure(): void
    {
        $this->loginUserByRole('admin');
        $id = '5a386e9f-6e7a-4ae7-9360-c8e529f78d93';
        $before = $this->ProductCategoryVariants->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryVariants',
            'action' => 'edit',
            $id,
        ];
        $data = [
            'name' => '',
            'product_category_id' => 'db4b4273-eddc-46d4-93c8-45cf7c6e058e',
            'enabled' => true,
        ];
        $this->put($url, $data);
        $this->assertResponseCode(200);
        $after = $this->ProductCategoryVariants->get($id);

        // assert save failed below
    }

    /**
    * Test delete method
    *
    * Tests the delete action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCategoryVariantsController::delete()
    * @throws Exception
    *
    * @return void
    */
    public function testDelete(): void
    {
        $cntBefore = $this->ProductCategoryVariants->find()->count();

        $this->loginUserByRole('admin');
        $id = '5a386e9f-6e7a-4ae7-9360-c8e529f78d93';

        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryVariants',
            'action' => 'delete',
            $id,
        ];
        $this->delete($url);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-category-variants');

        $cntAfter = $this->ProductCategoryVariants->find()->count();
        $this->assertEquals($cntBefore - 1, $cntAfter);
    }
}
